//
//  AppDelegate.swift
//  CGPathApply
//
//  Created by Gary Liddle on 05/03/2015.
//  Copyright (c) 2015 Gary Liddle. All rights reserved.
//

import Cocoa

@asmname("_rd_get_func_impl") // This is the only definition that works OOB with _rd_get_func_impl
func rd_get_func_impl(CFunctionPointer<(UnsafeMutablePointer<Void>, UnsafePointer<(CGPathElement)>) -> Void>)
    -> CFunctionPointer<(UnsafeMutablePointer<Void>, UnsafePointer<(CGPathElement)>) -> Void>;

/* I don't know how to return variable typed vars, so I can't make this work
@asmname("_rd_get_func_implT") func rd_get_func_implT<T>(T) -> T;
*/

@asmname("_rd_get_func_implT") // half way house between variable in and specific out
func rd_get_func_implT<T>(T)
    -> CFunctionPointer<(UnsafeMutablePointer<Void>, UnsafePointer<(CGPathElement)>) -> Void>;


@NSApplicationMain
class AppDelegate: NSObject, NSApplicationDelegate {

    @IBOutlet weak var window: NSWindow!
    @IBOutlet weak var btn_Draw: NSButton!
    @IBOutlet weak var vw_DrawView: NSImageView!
    
    func applicationDidFinishLaunching(aNotification: NSNotification) {
        // Insert code here to initialize your application
    }

    func applicationWillTerminate(aNotification: NSNotification) {
        // Insert code here to tear down your application
    }

    @IBAction func psh_btn_Draw(sender: AnyObject) {
        // The colour gradient ROYGBIV we want our line drawn in
        let aGradient = NSGradient(colorsAndLocations:
            (NSColor.redColor(), 0.0),
            (NSColor.orangeColor(), 0.166),
            (NSColor.yellowColor(), 0.33),
            (NSColor.greenColor(), 0.5),
            (NSColor.blueColor(), 0.67),
            (NSColor(red: 75.0, green: 0.0, blue: 130.0, alpha: 1.0), 0.834),
            (NSColor.purpleColor(), 1.0)
        );

        vw_DrawView.lockFocus();
        
        let v = vw_DrawView;
        plot(NSPoint(x:0, y:0), t: NSPoint(x: v!.frame.size.width, y: v!.frame.size.height), col: aGradient, width: 24.0)
        plot(NSPoint(x:0, y:v!.frame.size.height), t: NSPoint(x: v!.frame.size.width, y: 0), col: aGradient, width: 24.0)
        
        vw_DrawView.unlockFocus();
        
    }
    
    // draw a line from point 'f' to point 't', using colour gradient col
    func plot(f: NSPoint, t: NSPoint,col: NSGradient, width: CGFloat) {
        // The path onto which we draw the line
        let p = CGPathCreateMutable();
        
        let ctx = get_ctx();
        
        CGContextBeginPath(ctx);
        CGContextSetLineWidth(ctx, width);
        
        // any old colors used to create the line, as the gradient will eventually be used
        CGContextSetStrokeColorWithColor(ctx, NSColor.redColor().CGColor);
        CGContextSetFillColorWithColor(ctx, NSColor.greenColor().CGColor);
        
        // draw the line
        CGPathMoveToPoint(p, nil, f.x, f.y);
        CGPathAddLineToPoint(p, nil, t.x, t.y);
        CGPathCloseSubpath(p)
        CGContextAddPath(ctx, p);
        
        // create a path that can be filled with our gradient
        CGContextReplacePathWithStrokedPath(ctx);
        let cp = CGContextCopyPath(ctx);
        // convert the CGPath 'cp' to a bezier path 'np'
        let np = NSBezierPath();
        np.lineWidth = 0;
        let ConvertPathApplierFunctionPtr = getCFunctionPointer(ConvertPathApplierFunction);
        let npPtr = Ptr<NSBezierPath>(value: np);
        CGPathApply(cp, npPtr.cast(UnsafeMutablePointer<Void>), ConvertPathApplierFunctionPtr);
        
        // now draw the new bezier path using the colour gradient
        col.drawInBezierPath(np, angle: 45);
        
    }
    
    //func getCFunctionPointer<T>(swiftFunc: T) -> CFunctionPointer<T> { // can't get rd_get_func_implT to return T
    func getCFunctionPointer<T>(swiftFunc: T)
        -> CFunctionPointer<(UnsafeMutablePointer<Void>, UnsafePointer<(CGPathElement)>) -> Void>
    {
        let rawPointer = Ptr<T>(value: swiftFunc);
        
        let opaquePtr = COpaquePointer(rawPointer.ptr)
        let cFunPointer = CFunctionPointer<T>(opaquePtr);
        
        let realcFunPointer = rd_get_func_implT(cFunPointer);
        
        return realcFunPointer;
    }
    
    // get a context for CG to use when drawing
    func get_ctx() -> CGContext? {
        if let currentContext = NSGraphicsContext.currentContext() {
            var context: CGContext! = nil
            if (currentContext.respondsToSelector("CGContext")) {
                context = currentContext.CGContext
            } else {
                context = unsafeBitCast(currentContext.graphicsPort, CGContextRef.self);
            }
            return context
        }
        return nil
    }


}

