//
//  ConvertPathApplierFunctions.swift
//  CGPathApply
//
//  Created by Gary Liddle on 05/03/2015.
//  Copyright (c) 2015 Gary Liddle. All rights reserved.
//

import Foundation
import Cocoa

/*
 * A swift call back that can be used with CGPathApply and 
 * Dmitry Rodionov's _rd_get_func_impl function, to allow Swift callbacks from obj-c
 */
func ConvertPathApplierFunction( info: UnsafeMutablePointer<Void>, element:UnsafePointer<(CGPathElement)>) -> Void {
    
    var np:NSBezierPath? = nil;
    
    if (info != nil) {
        np = unsafeBitCast(info, UnsafeMutablePointer<NSBezierPath>.self).memory
    } else {
        return;
    }
    
    let e = element.memory
    
    switch (e.type.value) {
    case kCGPathElementMoveToPoint.value:
        np!.moveToPoint(e.points.memory);
        break;
    case kCGPathElementAddLineToPoint.value:
        np!.lineToPoint(e.points.memory);
        break;
    case kCGPathElementAddQuadCurveToPoint.value:
        let x = e.points[0];
        np!.curveToPoint(e.points[1]
            , controlPoint1: e.points[0]
            , controlPoint2: e.points[0]
        );
        break;
    case kCGPathElementAddCurveToPoint.value:
        np!.curveToPoint(e.points[2]
            , controlPoint1: e.points[0]
            , controlPoint2: e.points[1]
        );
        break;
    case kCGPathElementCloseSubpath.value:
        np!.closePath();
        break;
    default:
        break;
    }
}
